@@ -8,16 +8,22 @@ Copyright (c) 2012 TempoDB, Inc. All rights reserved.
 """
 
 from dateutil import parser
+import re
 import requests
 import simplejson
 import urllib
 import urllib2
 
+import tempodb
+
 
 API_HOST = 'api.tempo-db.com'
 API_PORT = 443
 API_VERSION = 'v1'
 
+VALID_SERIES_KEY = r'^[a-zA-Z0-9\.:;\-_]+$'
+RE_VALID_SERIES_KEY = re.compile(VALID_SERIES_KEY)
+
 
 class Database(object):
 
@@ -138,6 +144,9 @@ class Client(object):
         return series
 
     def create_series(self, key=None):
+        if key and not RE_VALID_SERIES_KEY.match(key):
+            raise ValueError("Series key must match the following regex: %s" % (VALID_SERIES_KEY,))
+
         params = {}
         if key is not None:
             params['key'] = key
@@ -184,6 +193,48 @@ class Client(object):
         series_val = series_key
         return self._read(series_type, series_val, start, end, interval, function)
 
+    def write_id(self, series_id, data):
+        series_type = 'id'
+        series_val = series_id
+        return self._write(series_type, series_val, data)
+
+    def write_key(self, series_key, data):
+        if series_key and not RE_VALID_SERIES_KEY.match(series_key):
+            raise ValueError("Series key must match the following regex: %s" % (VALID_SERIES_KEY,))
+
+        series_type = 'key'
+        series_val = series_key
+        return self._write(series_type, series_val, data)
+
+    def write_bulk(self, ts, data):
+        body = {
+            't': ts.isoformat(),
+            'data': data
+        }
+        json = self.request('/data/', method='POST', params=body)
+        return json
+
+    def increment_id(self, series_id, data):
+        series_type = 'id'
+        series_val = series_id
+        return self._increment(series_type, series_val, data)
+
+    def increment_key(self, series_key, data):
+        if series_key and not RE_VALID_SERIES_KEY.match(series_key):
+            raise ValueError("Series key must match the following regex: %s" % (VALID_SERIES_KEY,))
+
+        series_type = 'key'
+        series_val = series_key
+        return self._increment(series_type, series_val, data)
+
+    def increment_bulk(self, ts, data):
+        body = {
+            't': ts.isoformat(),
+            'data': data
+        }
+        json = self.request('/increment/', method='POST', params=body)
+        return json
+
     def _read(self, series_type, series_val, start, end, interval="", function=""):
         params = {
             'start': start.isoformat(),
@@ -204,42 +255,36 @@ class Client(object):
             return json
         return DataSet.from_json(json)
 
-    def write_id(self, series_id, data):
-        series_type = 'id'
-        series_val = series_id
-        return self.write(series_type, series_val, data)
-
-    def write_key(self, series_key, data):
-        series_type = 'key'
-        series_val = series_key
-        return self.write(series_type, series_val, data)
-
-    def write(self, series_type, series_val, data):
+    def _write(self, series_type, series_val, data):
         url = '/series/%s/%s/data/' % (series_type, series_val)
         body = [dp.to_json() for dp in data]
         json = self.request(url, method='POST', params=body)
         return json
 
-    def write_bulk(self, ts, data):
-        body = {
-            't': ts.isoformat(),
-            'data': data
-        }
-        json = self.request('/data/', method='POST', params=body)
+    def _increment(self, series_type, series_val, data):
+        url = '/series/%s/%s/increment/' % (series_type, series_val)
+        body = [dp.to_json() for dp in data]
+        json = self.request(url, method='POST', params=body)
         return json
 
     def request(self, target, method='GET', params={}):
         assert method in ['GET', 'POST', 'PUT'], "Only 'GET' and 'POST' are allowed for method."
 
+        headers = {
+            'User-Agent': 'tempodb-python/%s' % (tempodb.get_version(), )
+        }
+
         if method == 'POST':
+            headers['Content-Type'] = "application/json"
             base = self.build_full_url(target)
-            response = requests.post(base, data=simplejson.dumps(params), auth=(self.key, self.secret))
+            response = requests.post(base, data=simplejson.dumps(params), auth=(self.key, self.secret), headers=headers)
         elif method == 'PUT':
+            headers['Content-Type'] = "application/json"
             base = self.build_full_url(target)
-            response = requests.put(base, data=simplejson.dumps(params), auth=(self.key, self.secret))
+            response = requests.put(base, data=simplejson.dumps(params), auth=(self.key, self.secret), headers=headers)
         else:
             base = self.build_full_url(target, params)
-            response = requests.get(base, auth=(self.key, self.secret))
+            response = requests.get(base, auth=(self.key, self.secret), headers=headers)
 
         if response.status_code == 200:
             if response.text:
